package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.ApprovalRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/approvalRecord")
@Api(description = "企业历史投保记录")
public class ApprovalRecordController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ApprovalRecordService approvalRecordService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<ApprovalRecord> create(){
        MessageResult<ApprovalRecord> msgResult = new MessageResult<>();

        ApprovalRecord approvalRecord = new ApprovalRecord();

        msgResult.setData(approvalRecord);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ApprovalRecord> add(@RequestBody ApprovalRecord approvalRecord,@RequestAttribute String subject){
        MessageResult<ApprovalRecord> msgResult = new MessageResult<>();

        try {
            approvalRecord.setId(UUID.randomUUID().toString());
            approvalRecord.setDelFlag(false);
            approvalRecord.setCreateBy(subject);
            approvalRecord.setCreateTime(new Date());
            
            int affectCount = approvalRecordService.insert(approvalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(approvalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ApprovalRecord> edit(@PathVariable("id") String id){
        MessageResult<ApprovalRecord> msgResult = new MessageResult<>();

        try {
            ApprovalRecord approvalRecord = approvalRecordService.get(id);

            if (approvalRecord != null) {
                msgResult.setResult(true);
                msgResult.setData(approvalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ApprovalRecord> update(@RequestBody ApprovalRecord approvalRecord,@RequestAttribute String subject){
        MessageResult<ApprovalRecord> msgResult = new MessageResult<>();

        try {
            approvalRecord.setUpdateBy(subject);
            approvalRecord.setUpdateTime(new Date());
            
            int affectCount = approvalRecordService.update(approvalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(approvalRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            ApprovalRecord approvalRecord = approvalRecordService.get(id);
            approvalRecord.setDelFlag(true);
            approvalRecord.setUpdateBy(subject);
            approvalRecord.setUpdateTime(new Date());

            int affectCount = approvalRecordService.update(approvalRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                ApprovalRecord approvalRecord = approvalRecordService.get(id);
                approvalRecord.setDelFlag(true);
                approvalRecord.setUpdateBy(subject);
                approvalRecord.setUpdateTime(new Date());

                affectCount += approvalRecordService.update(approvalRecord);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<ApprovalRecord> page = approvalRecordService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
