package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.CompanyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/company")
@Api(description = "企业信息")
public class CompanyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyService companyService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Company> create(){
        MessageResult<Company> msgResult = new MessageResult<>();

        Company company = new Company();

        msgResult.setData(company);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<Company> add(@RequestBody Company company,@RequestAttribute String subject){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            company.setId(UUID.randomUUID().toString());
            company.setDelFlag(false);
            company.setCreateBy(subject);
            company.setCreateTime(new Date());
            
            int affectCount = companyService.insert(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Company> edit(@PathVariable("id") String id){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(id);

            if (company != null) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<Company> update(@RequestBody Company company,@RequestAttribute String subject){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());
            
            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(id);
            company.setDelFlag(true);
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());

            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Company company = companyService.get(id);
                company.setDelFlag(true);
                company.setUpdateBy(subject);
                company.setUpdateTime(new Date());

                affectCount += companyService.update(company);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<Company> page = companyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
