package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/insuranceDefinitionLimit")
@Api(description = "险种限额信息")
public class InsuranceDefinitionLimitController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceDefinitionLimit> create(){
        MessageResult<InsuranceDefinitionLimit> msgResult = new MessageResult<>();

        InsuranceDefinitionLimit insuranceDefinitionLimit = new InsuranceDefinitionLimit();

        msgResult.setData(insuranceDefinitionLimit);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceDefinitionLimit> add(@RequestBody InsuranceDefinitionLimit insuranceDefinitionLimit,@RequestAttribute String subject){
        MessageResult<InsuranceDefinitionLimit> msgResult = new MessageResult<>();

        try {
            insuranceDefinitionLimit.setId(UUID.randomUUID().toString());
            insuranceDefinitionLimit.setDelFlag(false);
            insuranceDefinitionLimit.setCreateBy(subject);
            insuranceDefinitionLimit.setCreateTime(new Date());
            
            int affectCount = insuranceDefinitionLimitService.insert(insuranceDefinitionLimit);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinitionLimit);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceDefinitionLimit> edit(@PathVariable("id") String id){
        MessageResult<InsuranceDefinitionLimit> msgResult = new MessageResult<>();

        try {
            InsuranceDefinitionLimit insuranceDefinitionLimit = insuranceDefinitionLimitService.get(id);

            if (insuranceDefinitionLimit != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinitionLimit);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceDefinitionLimit> update(@RequestBody InsuranceDefinitionLimit insuranceDefinitionLimit,@RequestAttribute String subject){
        MessageResult<InsuranceDefinitionLimit> msgResult = new MessageResult<>();

        try {
            insuranceDefinitionLimit.setUpdateBy(subject);
            insuranceDefinitionLimit.setUpdateTime(new Date());
            
            int affectCount = insuranceDefinitionLimitService.update(insuranceDefinitionLimit);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinitionLimit);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceDefinitionLimit insuranceDefinitionLimit = insuranceDefinitionLimitService.get(id);
            insuranceDefinitionLimit.setDelFlag(true);
            insuranceDefinitionLimit.setUpdateBy(subject);
            insuranceDefinitionLimit.setUpdateTime(new Date());

            int affectCount = insuranceDefinitionLimitService.update(insuranceDefinitionLimit);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceDefinitionLimit insuranceDefinitionLimit = insuranceDefinitionLimitService.get(id);
                insuranceDefinitionLimit.setDelFlag(true);
                insuranceDefinitionLimit.setUpdateBy(subject);
                insuranceDefinitionLimit.setUpdateTime(new Date());

                affectCount += insuranceDefinitionLimitService.update(insuranceDefinitionLimit);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsuranceDefinitionLimit> page = insuranceDefinitionLimitService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
