package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.TransactionRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/transactionRecord")
@Api(description = "企业账户流水信息")
public class TransactionRecordController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private TransactionRecordService transactionRecordService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<TransactionRecord> create(){
        MessageResult<TransactionRecord> msgResult = new MessageResult<>();

        TransactionRecord transactionRecord = new TransactionRecord();

        msgResult.setData(transactionRecord);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<TransactionRecord> add(@RequestBody TransactionRecord transactionRecord,@RequestAttribute String subject){
        MessageResult<TransactionRecord> msgResult = new MessageResult<>();

        try {
            transactionRecord.setId(UUID.randomUUID().toString());
            transactionRecord.setDelFlag(false);
            transactionRecord.setCreateBy(subject);
            transactionRecord.setCreateTime(new Date());
            
            int affectCount = transactionRecordService.insert(transactionRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(transactionRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<TransactionRecord> edit(@PathVariable("id") String id){
        MessageResult<TransactionRecord> msgResult = new MessageResult<>();

        try {
            TransactionRecord transactionRecord = transactionRecordService.get(id);

            if (transactionRecord != null) {
                msgResult.setResult(true);
                msgResult.setData(transactionRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<TransactionRecord> update(@RequestBody TransactionRecord transactionRecord,@RequestAttribute String subject){
        MessageResult<TransactionRecord> msgResult = new MessageResult<>();

        try {
            transactionRecord.setUpdateBy(subject);
            transactionRecord.setUpdateTime(new Date());
            
            int affectCount = transactionRecordService.update(transactionRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(transactionRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            TransactionRecord transactionRecord = transactionRecordService.get(id);
            transactionRecord.setDelFlag(true);
            transactionRecord.setUpdateBy(subject);
            transactionRecord.setUpdateTime(new Date());

            int affectCount = transactionRecordService.update(transactionRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                TransactionRecord transactionRecord = transactionRecordService.get(id);
                transactionRecord.setDelFlag(true);
                transactionRecord.setUpdateBy(subject);
                transactionRecord.setUpdateTime(new Date());

                affectCount += transactionRecordService.update(transactionRecord);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<TransactionRecord> page = transactionRecordService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
