package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.CompanyMember;
import com.jpsoft.picc.modules.base.service.CompanyMemberService;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@Api(description="企业人员管理")
@RestController
@RequestMapping("/auth/companyMember")
public class CompanyMemberController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyMemberService companyMemberService;

    @ApiOperation(value="读取企业人员列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业ID", required = true, paramType = "form")
    })
    public MessageResult<Map> list(
            String id,
            @RequestParam(value="companyId",defaultValue="") String companyId,
            @RequestAttribute String subject){
        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("delFlag",false);

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("companyId",id);
        }

        Page<CompanyMember> page = companyMemberService.pageSearch(searchParams,1,1000,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="读取企业人员分页列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form")
    })
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id",id);
        }

        Page<CompanyMember> page = companyMemberService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="添加企业人员")
    @PostMapping("add")
    public MessageResult<CompanyMember> add(@RequestBody CompanyMember companyMember, @RequestAttribute String subject){
        MessageResult<CompanyMember> msgResult = new MessageResult<>();

        try {
            companyMember.setId(UUID.randomUUID().toString());
            companyMember.setDelFlag(false);
            companyMember.setCreateBy(subject);
            companyMember.setCreateTime(new Date());

            int affectCount = companyMemberService.insert(companyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(companyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取企业人员")
    @GetMapping("edit")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form")
    })
    public MessageResult<CompanyMember> edit(@RequestParam(value="编号",defaultValue="") String id){
        MessageResult<CompanyMember> msgResult = new MessageResult<>();

        try {
            CompanyMember companyMember = companyMemberService.get(id);

            if (companyMember != null) {
                msgResult.setResult(true);
                msgResult.setData(companyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新企业人员")
    @PostMapping("update")
    public MessageResult<CompanyMember> update(@RequestBody CompanyMember companyMember,@RequestAttribute String subject){
        MessageResult<CompanyMember> msgResult = new MessageResult<>();

        try {
            companyMember.setUpdateBy(subject);
            companyMember.setUpdateTime(new Date());

            int affectCount = companyMemberService.update(companyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(companyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="删除企业人员")
    @PostMapping("delete")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form")
    })
    public MessageResult<Integer> delete(@RequestParam(value="编号",defaultValue="") String id,
                                         @RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            CompanyMember companyMember = companyMemberService.get(id);
            companyMember.setDelFlag(true);
            companyMember.setUpdateBy(subject);
            companyMember.setUpdateTime(new Date());

            int affectCount = companyMemberService.update(companyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
