package com.jpsoft.picc.modules.pub.controller;

import com.alibaba.druid.util.StringUtils;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import io.swagger.annotations.*;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

@CrossOrigin
@Controller
@Api(description = "用户登录、登出")
public class UserController {
    @Value("${cas.server-url-prefix}")
    private String serverUrlPrefix;

    @Value("${cas.server-login-url}")
    private String serverLoginUrl;

    @PostMapping(value="/login")
    @ResponseBody
    @ApiOperation(value="登录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userName",value = "用户名", required = true, paramType = "form"),
            @ApiImplicitParam(name = "password",value = "密码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "service",value = "跳转地址", required = true, paramType = "form")
    })
    public MessageResult<Map> login(
            String userName,String password,
            String service,HttpSession session) throws Exception{
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            String casServerTicketUrl = serverUrlPrefix + "/v1/tickets";

            String serviceTicketUrl = getTGT(casServerTicketUrl, userName, password, service);

            if (StringUtils.isEmpty(serviceTicketUrl)) {
                msgResult.setResult(false);
            } else {
                Map<String, Object> dataMap = new HashMap<>();

                //获取TGT
                String tgt = serviceTicketUrl.substring(casServerTicketUrl.length() + 1);

                session.setAttribute("tgt",tgt);

                // 采用st只能显示单应用登录
//                String st = getST(serviceTicketUrl, service);
                String serviceEnc = "";

                try {
                    serviceEnc = URLEncoder.encode(service, "UTF-8");
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                /**
                 * 注意
                 * 1.cas-server中ticketExpirationPolicies.xml中时间要设长，否则手工测试时，会失效
                 * 2.这里redirect的url，要与service一致
                 * 3.如果是前后端分离的应用，可以要前端先通过ajax请求该方法获取serviceEnc和tgt然后做跳转，就可以在客户端写入tgc
                 */
//                dataMap.put("redirectUrl", service + "?ticket=" + st);
                dataMap.put("redirectUrl",serverLoginUrl + "?service=" + serviceEnc + "&tgt=" + tgt);
                dataMap.put("ticket", tgt);

                msgResult.setData(dataMap);
                msgResult.setResult(true);
            }
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return  msgResult;
    }

    /**
     * cas登录后会带st返回该url，根据st 拦截器会自动到cas服务端注册后再次返回该url，
     * 这时就可以正常访问前端，前端和后端需要处于同一域下，以共享cookie。
     * @return
     */
    @GetMapping(value="/auth/portal")
    @ApiOperation(value="返回前端")
    public String portal(){
        return "redirect:/portal/";
    }

    @GetMapping(value="/auth/userInfo")
    @ResponseBody
    @CrossOrigin
    @ApiOperation(value="获取用户信息")
    public MessageResult<Map> userInfo(HttpServletRequest request, HttpServletResponse response) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

            Map<String, Object> map = new HashMap<>();

            for (String key : principal.getAttributes().keySet()) {
                map.put(key, principal.getAttributes().get(key));
            }

            msgResult.setData(map);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @GetMapping("/logout")
    @ResponseBody
    @ApiOperation(value="退出")
    public MessageResult<String> logout(HttpSession session){
        MessageResult<String> msgResult = new MessageResult<>();

        String tgt = (String)session.getAttribute("tgt");
        String casServerTicketUrl = serverUrlPrefix + "/v1/tickets";

        HttpClient client = new HttpClient();
        DeleteMethod del = new DeleteMethod(casServerTicketUrl + "/" + tgt);

        try {
            client.executeMethod(del);
            String html = del.getResponseBodyAsString();

            System.out.println(html);

            msgResult.setResult(true);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();

            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
        }

        return msgResult;
    }

    private String getTGT(String casServerTicketUrl,String username,String password,String service){
        String url = "";

        HttpClient client = new HttpClient();

        PostMethod post = new PostMethod(casServerTicketUrl);

        post.setRequestBody(
                new NameValuePair[] {
                        new NameValuePair("username", username),
                        new NameValuePair("password", password)
//					new NameValuePair("service", service)
                });

        try {
            client.executeMethod(post);
            String html = post.getResponseBodyAsString();

            //System.out.println(html);
            Document doc = Jsoup.parse(html);

            url = doc.select("form").attr("action");

            System.out.println("TGT=" + url);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        finally{
            post.releaseConnection();
        }

        return url;
    }

    public String getST(String url,String service){
        String serviceTicket = "";

        HttpClient client = new HttpClient();

        PostMethod post = new PostMethod(url);

        post.setRequestBody(
                new NameValuePair[] {
                        new NameValuePair("service", service)
                });

        try {
            client.executeMethod(post);
            serviceTicket = post.getResponseBodyAsString();

            System.out.println("ST=" + serviceTicket);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        finally{
            post.releaseConnection();
        }

        return serviceTicket;
    }

    private void ticketValidate(String serverValidate, String serviceTicket, String service) {
        final HttpClient client = new HttpClient();

        GetMethod post = null;

        try {
            String url = serverValidate+"?"+"ticket="+serviceTicket+"&service="+ URLEncoder.encode(service, "UTF-8");

            System.out.println("url=" + url);

            post = new GetMethod(url);
            client.executeMethod(post);

            final String response = post.getResponseBodyAsString();

            System.out.println(response);
        }
        catch (Exception e) {

        } finally {
            //释放资源
            post.releaseConnection();
        }
    }
}
